/* -------------------------------------------------------------------
  Titre: Serveur HTTP V4 (dossier GENERIQUE)
  Date : 9/5/2023 - mise à jour des bibliothèques
  Objectifs:
  Commander la led L et les sorties D0, D1 d'une carte MKR Wifi 1010,
  et obtenir les valeurs présentes sur les entrées A0, A1,A2 et A3 à 
  partir d'un navigateur (requêtes GET ou POST).
  URL reconnues:
  - http://@IP renvoie "La page d'accueil"
  - http://@IP/lire/val1 demande la valeur sur A0
  - http://@IP/lire/val2 demande la valeur sur A1
  - http://@IP/lire/val3 demande la valeur sur A2
  - http://@IP/lire/val4 demande la valeur sur A3
  - http://@IP/lire/vals1et2 demande les valeurs sur les entrées A0 et A1 (json)
  - http://@IP/ecrire/valA active ou désactive la LED L et D0
  - http://@IP/ecrire/valB active ou désactive D1
  Bibliothèques :
  - WiFiWebServer, WIFININA, SPI : appelées dans defines.h
  Documentation de WiFiWebServer et WifiNINA
  - https://github.com/khoih-prog/WiFiWebServer
  - https://www.arduino.cc/en/Reference/WiFiNINA
  Carte arduino:
  - Module NINA (Arduino MKR WiFi 1010, MKR VIDOR 4000 et UNO WiFi Rev.2)
  Fichiers
  - define.h, Server_HTTP_V4
  -----------------------------------------------------------------------*/
// Définitions
#define LED LED_BUILTIN //  LED_BUILTIN (L sur la carte !)
#define D0 0
#define D1 1

// Bibliothèques à ajouter
#include "defines.h"

// Page d'accueil
const String paccueil =
  F("<html>\
<head><meta charset=\"utf-8\">\
<title>Serveur MKR Wifi 1010 V4</title>\
<style>\
.ip{color:blue;}\
</style>\
</head>\
<body>\
<h3>[Accueil] serveur HTTP V4 (MKR Wifi 1010)</h3>\
<strong>Commandes reconnues:</strong><br>\
<em>Activation de (la LED L et de la sortie D0) <- valA ou de la sortie D1 <- valB</em>\
<ul>\
<li>- http://@IP<span class=\"ip\"><a href=\"/ecrire/valA?val=1\">/ecrire/valA?val=1</a></span></li>\
<li>- http://@IP<span class=\"ip\"><a href=\"/ecrire/valB?val=1\">/ecrire/valB?val=1</a></span></li>\
</ul>\
<em>Désactivation de (la LED L et de la sortie D0) <- valA ou de la sortie D1 <- valB</em>\
<ul>\
<li>- http://@IP<span class=\"ip\"><a href=\"/ecrire/valA?val=0\">/ecrire/valA?val=0</a></span></li>\
<li>- http://@IP<span class=\"ip\"><a href=\"/ecrire/valB?val=0\">/ecrire/valB?val=0</a></span></li>\
</ul>\
<em>Lecture des entrées analogiques A0 -> va11 et A1 -> val2</em>\
<ul>\
<li>- http://@IP<span class=\"ip\"><a href=\"/lire/val1\">/lire/val1</a></span></li>\
<li>- http://@IP<span class=\"ip\"><a href=\"/lire/val2\">/lire/val2</a></span></li>\
<li>- http://@IP<span class=\"ip\"><a href=\"/lire/val3\">/lire/val3</a></span></li>\
<li>- http://@IP<span class=\"ip\"><a href=\"/lire/val4\">/lire/val4</a></span></li>\
<li>- http://@IP<span class=\"ip\"><a href=\"/lire/vals1et2\">/lire/vals1et2</a></span> [JSON]</li>\
</ul>\
</body></html>");

// Paramètres réseau, cryptage WPA
const char ssid[] = "SynBoxLAN"; // nom du SSID
const char pass[] = "12345678"; // mot de passe du réseau

// Création du serveur HTTP
int status = WL_IDLE_STATUS;
WiFiWebServer server(80);

// Gestionnaires de requête
//----------------------------------------------------------------
// g1. Page d'accueil
void handleRoot() {
  server.send(200, F("text/html"), paccueil); // Réponse
}

// g2. Commande de la led L de la carte et de la sortie D0
void ecritureValA() {
  if (server.arg(0) == "0") { // Test de la valeur du premier argument
    digitalWrite(LED, LOW); // Désactivation de la commande de la LED L
    digitalWrite(D0, LOW);  // et de la sortie D0
    server.send(200, F("text/plain"), String(server.arg(0)));
  }
  else if (server.arg(0) == "1")
  {
    digitalWrite(LED, HIGH); // Activation de la commande de la LED L
    digitalWrite(D0, HIGH);  // et de la sortie D0
    server.send(200, F("text/plain"), String(server.arg(0)));
  }
  else {
    server.send(200, F("text/plain"), String("Commande <-  0 ou 1 !!!"));
  }
}

// g3. Commande de la sortie D1
void ecritureValB() {
  if (server.arg(0) == "0") { // Test de la valeur du premier argument
    digitalWrite(D1, LOW);  // Désactivation de la sortie D1
    server.send(200, F("text/plain"), String(server.arg(0)));
  }
  else if (server.arg(0) == "1")
  {
    digitalWrite(D1, HIGH);  // Activation de la sortie D1
    server.send(200, F("text/plain"), String(server.arg(0)));
  }
  else {
    server.send(200, F("text/plain"), String("Commande <-  0 ou 1 !!!"));
  }
}

// g4. Mesure et transmission de la valeur analogique présente sur A0
void lectureVal1() {
  unsigned int val1 = analogRead(0);
  server.send(200, F("text/plain"), String(val1));
}

// g5. Mesure et transmission de la valeur analogique présente sur A1
void lectureVal2() {
  unsigned int val2 = analogRead(1);
  server.send(200, F("text/plain"), String(val2));
}

// g6. Mesure et transmission de la valeur analogique présente sur A0
void lectureVal3() {
  unsigned int val3 = analogRead(2);
  server.send(200, F("text/plain"), String(val3));
}

// g7. Mesure et transmission de la valeur analogique présente sur A1
void lectureVal4() {
  unsigned int val4 = analogRead(3);
  server.send(200, F("text/plain"), String(val4));
}

// g8. Mesure et transmission des valeurs analogiques présentes sur A0 et A1 au format JSON
void lectureVals1et2()
{
  unsigned int val1 = analogRead(0);
  unsigned int val2 = analogRead(1);
  String msgjson = "[{\"val1\":";
  msgjson += val1;
  msgjson += ",";
  msgjson += "\"val2\":";
  msgjson += val2;
  msgjson += "}]";
  server.send(200, F("text/plain"), msgjson);
}

// g9. Traitement des requêtes non prises en charge
void handleNotFound()
{
  String message = F("File Not Found\n\n");
  message += F("URI: ");
  message += server.uri();
  message += F("\nMethod: ");
  message += (server.method() == HTTP_GET) ? F("GET") : F("POST");
  message += F("\nArguments: ");
  message += server.args();
  message += F("\n");
  for (uint8_t i = 0; i < server.args(); i++)
  {
    message += " " + server.argName(i) + ": " + server.arg(i) + "\n";
  }
  server.send(404, F("text/plain"), message);
}
//----------------------------------------------------------------

void setup()
{
  // Configuration des E/S
  pinMode(LED, OUTPUT); // Led L de la carte
  pinMode(D0, OUTPUT);
  pinMode(D1, OUTPUT);
  Serial.begin(115200); // Moniteur pour la mise au point

  // 1. Vérification de la connexion au module Wifi
  if (WiFi.status() == WL_NO_MODULE)
  {
    Serial.println(" La communication avec le module WiFi a échoué!");
    // ATTENTION : blocage du programme !!!!
    while (true)
      ;
  }

  // 2. Connexion au réseau Wifi
  while (status != WL_CONNECTED)
  {
    Serial.print("Tentative de connexion au réseau: ");
    Serial.println(ssid);
    status = WiFi.begin(ssid, pass); // si WPA, WPA2
    // Attente de 10s avant la prochaine tentative de connexion
    delay(10000);
  }

  // 3. Connecté, inscription des gestionnaires
  server.on(F("/"), handleRoot);
  server.on(F("/lire/val1"), lectureVal1);
  server.on(F("/lire/val2"), lectureVal2);
  server.on(F("/lire/val3"), lectureVal3);
  server.on(F("/lire/val4"), lectureVal4);
  server.on(F("/lire/vals1et2"), lectureVals1et2);
  server.on(F("/ecrire/valA"), ecritureValA);
  server.on(F("/ecrire/valB"), ecritureValB);

  server.onNotFound(handleNotFound);

  // et démarrage du serveur Web sur le port défini
  // lors de l'initialisation
  server.begin();
}

void loop()
{
  server.handleClient();
}
